'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs_Observable = require('rxjs/Observable');
var jwt = require('jsonwebtoken');
var tsDbHelper = require('ts-db-helper');
var express = require('express');
var bodyParser = require('body-parser');
var http = require('http');
require('rxjs/add/operator/finally');
require('rxjs/add/operator/switchMap');
require('rxjs/add/Observable/empty');

var PermissionResult = /** @class */ (function () {
    function PermissionResult(ok) {
        this.isOk = ok || false;
    }
    return PermissionResult;
}());

var IsAutenticatedPermission = /** @class */ (function () {
    function IsAutenticatedPermission() {
    }
    IsAutenticatedPermission.prototype.check = function (req, userModel) {
        return rxjs_Observable.Observable.create(function (observer) {
            observer.next(new PermissionResult((!!req.user) && !req.user.isAnonymous));
            observer.complete();
        });
    };
    return IsAutenticatedPermission;
}());

var AnonymousPermission = /** @class */ (function () {
    function AnonymousPermission() {
    }
    AnonymousPermission.prototype.check = function (req, userModel) {
        return rxjs_Observable.Observable.create(function (observer) {
            observer.next(new PermissionResult(true));
            observer.complete();
        });
    };
    return AnonymousPermission;
}());

var NoAuthFilter = /** @class */ (function () {
    function NoAuthFilter() {
    }
    NoAuthFilter.prototype.filter = function (req, userModel) {
        return rxjs_Observable.Observable.create(function (observer) {
            var user = new userModel();
            user.isAnonymous = true;
            observer.next(user);
            observer.complete();
        });
    };
    return NoAuthFilter;
}());

var NoAuthenticator = /** @class */ (function () {
    function NoAuthenticator() {
        this.authFilter = new NoAuthFilter();
    }
    NoAuthenticator.prototype.authenticate = function (req, resp, userModel) {
        resp.status(204);
        resp.send(null);
    };
    return NoAuthenticator;
}());

var AuthFilterError = /** @class */ (function () {
    function AuthFilterError(message) {
        this.message = message;
        this.name = 'Auth filter error';
    }
    return AuthFilterError;
}());

var StringUtils = /** @class */ (function () {
    function StringUtils() {
    }
    StringUtils.isString = function (variable) {
        return typeof variable === 'string' || variable instanceof String;
    };
    return StringUtils;
}());

var Log = /** @class */ (function () {
    function Log() {
    }
    Object.defineProperty(Log, "logger", {
        get: function () {
            if (!Log.loggerValue) {
                Log.loggerValue = tsDbHelper.QueryManager.getInstance().logger;
            }
            return Log.loggerValue;
        },
        enumerable: true,
        configurable: true
    });
    Log.init = function (logger) {
        this.loggerValue = logger;
    };
    Log.l = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        (_a = Log.logger).log.apply(_a, args);
        var _a;
    };
    Log.d = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        (_a = Log.logger).debug.apply(_a, args);
        var _a;
    };
    Log.t = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        (_a = Log.logger).trace.apply(_a, args);
        var _a;
    };
    Log.i = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        (_a = Log.logger).info.apply(_a, args);
        var _a;
    };
    Log.w = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        (_a = Log.logger).warn.apply(_a, args);
        var _a;
    };
    Log.e = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        (_a = Log.logger).error.apply(_a, args);
        var _a;
    };
    Log.f = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        (_a = Log.logger).fatal.apply(_a, args);
        var _a;
    };
    return Log;
}());

var JWTFilter = /** @class */ (function () {
    function JWTFilter(config) {
        this.config = config;
    }
    JWTFilter.prototype.filter = function (req, userModel) {
        var _this = this;
        return rxjs_Observable.Observable.create(function (observer) {
            var header = req.headers[_this.config.httpHeader.toLowerCase()];
            var token = null;
            var err = null;
            if (header && StringUtils.isString(header)) {
                if (_this.config.httpHeaderAuthType) {
                    var headerParts = header.split(' ');
                    if (headerParts.length === 2 && headerParts[0].toUpperCase() === _this.config.httpHeaderAuthType.toUpperCase()) {
                        token = headerParts[1];
                    }
                    else {
                        err = 'Token header do not conform to "' + _this.config.httpHeaderAuthType + ' <token>"';
                    }
                }
                else {
                    token = header;
                }
            }
            if (token) {
                var decodeConfig = {};
                if (_this.config.audience) {
                    decodeConfig.audience = _this.config.audience;
                }
                if (_this.config.algorithm) {
                    decodeConfig.algorithm = _this.config.algorithm;
                }
                if (_this.config.issuer) {
                    decodeConfig.algorithm = _this.config.algorithm;
                }
                jwt.verify(token, _this.config.secret, decodeConfig, function (error, decoded) {
                    if (error) {
                        Log.e(error.message || JSON.stringify(error));
                        observer.error(new AuthFilterError(error.name + ': ' + error.message));
                    }
                    else if (decoded instanceof String) {
                        observer.error(new AuthFilterError('Token has an invalid payload'));
                    }
                    else if (decoded.user) {
                        try {
                            var user = _this.config.deserializeUser(decoded.user);
                            observer.next(user);
                        }
                        catch (e) {
                            Log.e(e.message || JSON.stringify(e));
                            observer.error(new AuthFilterError('Token contains invalid user'));
                        }
                    }
                    observer.complete();
                });
            }
            else {
                if (err) {
                    observer.error(new AuthFilterError(err));
                }
                else {
                    var user = new userModel();
                    user.isAnonymous = true;
                    observer.next(user);
                }
                observer.complete();
            }
        });
    };
    return JWTFilter;
}());

var JWTConfiguration = /** @class */ (function () {
    function JWTConfiguration() {
        this.expiresIn = 60 * 60 * 24;
        this.httpHeader = 'Authorization';
        this.httpHeaderAuthType = 'JWT';
        this.issuer = 'trf-app:issuer';
        this.audience = 'trf-app:audience';
    }
    return JWTConfiguration;
}());

var JWTAuthenticator = /** @class */ (function () {
    function JWTAuthenticator(config) {
        this.config = config;
        this.authFilter = new JWTFilter(this.config);
    }
    JWTAuthenticator.prototype.authenticate = function (req, resp, userModel) {
        var _this = this;
        var body = req.body;
        if (body && body.username && body.password) {
            userModel.authenticate(body.username, body.password).subscribe(function (user) {
                if (!user || user.isAnonymous) {
                    resp.status(400);
                    resp.json({
                        reason: 'Invalid credentials'
                    });
                }
                else {
                    var userData = _this.config.serializeUser(user);
                    var config = {};
                    if (_this.config.algorithm) {
                        config.algorithm = _this.config.algorithm;
                    }
                    if (_this.config.audience) {
                        config.audience = _this.config.audience;
                    }
                    if (_this.config.issuer) {
                        config.issuer = _this.config.issuer;
                    }
                    try {
                        var token = jwt.sign({ user: userData }, _this.config.secret, config);
                        resp.status(200);
                        resp.json({
                            token: token
                        });
                    }
                    catch (err) {
                        resp.status(500);
                        resp.json({
                            reason: 'Token creation did fail',
                            details: err.message || JSON.stringify(err)
                        });
                    }
                }
            }, function (err) {
                var content = { reason: '' };
                if (err instanceof String) {
                    content.reason = err;
                }
                else if (err && err.message) {
                    content.reason = err.message;
                }
                else {
                    content.reason = JSON.stringify(err);
                }
                resp.status(500);
                resp.json(content);
            });
        }
        else {
            resp.status(400);
            resp.json({
                reason: 'Username, password or both are missing in message body'
            });
        }
    };
    return JWTAuthenticator;
}());

var Credentials = /** @class */ (function () {
    function Credentials() {
    }
    return Credentials;
}());
var BasicAuthFilter = /** @class */ (function () {
    function BasicAuthFilter() {
    }
    BasicAuthFilter.prototype.filter = function (req, userModel) {
        var _this = this;
        return rxjs_Observable.Observable.create(function (observer) {
            try {
                var credentials = _this.getCredentials(req.headers.authorization);
                if (credentials) {
                    userModel.authenticate(credentials.login, credentials.password).subscribe(function (user) { return observer.next(user); }, function (err) { return observer.error(new AuthFilterError((err && (err.message || JSON.stringify(err)) || ''))); }, function () { return observer.complete(); });
                }
                else {
                    var user = new userModel();
                    user.isAnonymous = true;
                    observer.next(user);
                    observer.complete();
                }
            }
            catch (e) {
                observer.error(e);
                observer.complete();
            }
        });
    };
    BasicAuthFilter.prototype.getCredentials = function (authorization) {
        var credentials = null;
        if (authorization instanceof String) {
            var tokens = authorization.split(' ');
            if (tokens.length === 2 && tokens[0].toUpperCase() === 'BASIC' && tokens[1]) {
                var buf = new Buffer(tokens[1], 'base64');
                var plain_auth = buf.toString();
                var creds = plain_auth.split(':');
                if (creds && creds.length === 2) {
                    credentials = new Credentials();
                    credentials.login = creds[0];
                    credentials.password = creds[1];
                }
                else {
                    throw new AuthFilterError('Found authorization content with invalid encoded datas');
                }
            }
            else {
                throw new AuthFilterError('Found authorization content with invalid content');
            }
        }
        return credentials;
    };
    return BasicAuthFilter;
}());

var BasicAuthAuthenticator = /** @class */ (function () {
    function BasicAuthAuthenticator() {
        this.authFilter = new BasicAuthFilter();
    }
    BasicAuthAuthenticator.prototype.authenticate = function (req, resp, userModel) {
        this.authFilter.filter(req, userModel).subscribe(function (user) {
            if (user && !user.isAnonymous) {
                resp.status(204);
                resp.send();
            }
            else {
            }
        }, function (err) {
            resp.status(400);
            resp.json({
                reason: 'invalid authorization header'
            });
        });
    };
    return BasicAuthAuthenticator;
}());

var IUser = /** @class */ (function () {
    function IUser() {
    }
    return IUser;
}());

var RouteManager = /** @class */ (function () {
    function RouteManager() {
        this.bundles = {};
        this.namespacies = {};
    }
    RouteManager.getInstance = function () {
        if (!RouteManager.instance) {
            RouteManager.instance = new RouteManager();
        }
        return RouteManager.instance;
    };
    RouteManager.prototype.addRouteBundle = function (viewSet, bundle) {
        if (this.bundles.hasOwnProperty(viewSet.name || viewSet.constructor.name)) {
            throw new Error('Duplicate adding viewSet to route manager');
        }
        this.bundles[viewSet.name || viewSet.constructor.name] = bundle;
    };
    RouteManager.prototype.getRouteBundle = function (viewSet) {
        return this.bundles[viewSet.name || viewSet.constructor.name];
    };
    RouteManager.prototype.addNamespaceConfiguration = function (namespace, config) {
        if (this.bundles.hasOwnProperty(namespace.name)) {
            throw new Error('Duplicate adding viewSet to route manager');
        }
        this.namespacies[namespace.name] = config || null;
    };
    RouteManager.prototype.getNamespaceConfiguration = function (namespace) {
        var config = this.namespacies[namespace.name];
        if (config) {
            return config;
        }
        else {
            throw new Error('Namespace "' + namespace.name + '" is not correctly declared, did you use @Namespace ?');
        }
    };
    RouteManager.prototype.isRegisteredNamespace = function (namespace) {
        return this.namespacies.hasOwnProperty(namespace.name);
    };
    RouteManager.prototype.isRegisteredViewset = function (viewset) {
        return this.bundles.hasOwnProperty(viewset.name);
    };
    return RouteManager;
}());

var RouteBundle = /** @class */ (function () {
    function RouteBundle() {
        this.routes = {};
    }
    return RouteBundle;
}());

function Route(config) {
    return function (target) {
        var bundle = RouteManager.getInstance().getRouteBundle(target);
        if (!bundle) {
            bundle = new RouteBundle();
            RouteManager.getInstance().addRouteBundle(target, bundle);
        }
        bundle.config = config;
    };
}

function Namespace(config) {
    return function (constructor) {
        RouteManager.getInstance().addNamespaceConfiguration(constructor, config);
        return constructor;
    };
}

function endPointFactory(method, config) {
    return function (target, propertyKey, descriptor) {
        var bundle = RouteManager.getInstance().getRouteBundle(target);
        if (!bundle) {
            bundle = new RouteBundle();
            RouteManager.getInstance().addRouteBundle(target, bundle);
        }
        config.method = method;
        bundle.routes[propertyKey] = config;
    };
}
function Get(config) {
    return endPointFactory('get', config);
}
function Post(config) {
    return endPointFactory('post', config);
}
function Put(config) {
    return endPointFactory('put', config);
}
function Patch(config) {
    return endPointFactory('patch', config);
}
function Delete$1(config) {
    return endPointFactory('delete', config);
}
function Options(config) {
    return endPointFactory('options', config);
}
function Head(config) {
    return endPointFactory('head', config);
}
function Connect(config) {
    return endPointFactory('connect', config);
}

function All(config) {
    return endPointFactory('all', config);
}

var RouteConfigurator = /** @class */ (function () {
    function RouteConfigurator() {
    }
    return RouteConfigurator;
}());

var NamespaceConfigurator = /** @class */ (function () {
    function NamespaceConfigurator() {
        this.views = [];
    }
    return NamespaceConfigurator;
}());

var EndPointConfigurator = /** @class */ (function () {
    function EndPointConfigurator() {
    }
    return EndPointConfigurator;
}());

var Page = /** @class */ (function () {
    function Page() {
    }
    Page.prototype.isLast = function () {
        return this.totalElementsCount <= (this.number + 1) * this.size;
    };
    return Page;
}());

var ViewSet = /** @class */ (function () {
    function ViewSet() {
        if (!this.query) {
            this.query = tsDbHelper.Select(this.model).setSize(50);
        }
        if (!this.serializer) {
            this.serializer = tsDbHelper.AutoSerializer;
        }
    }
    ViewSet.prototype.search = function (req, resp) {
    };
    ViewSet.prototype.list = function (req, resp) {
        var _this = this;
        var is400Sent = false;
        if (req.query.size) {
            var size = parseInt(req.query.pageSize, 10);
            if (isNaN(size)) {
                resp.status(400);
                resp.json({
                    reason: 'Invalid page size'
                });
                is400Sent = true;
            }
            else {
                this.query.setSize(size);
            }
        }
        if (req.query.page) {
            var page = parseInt(req.query.pageNumber, 10);
            if (isNaN(page)) {
                resp.status(400);
                resp.json({
                    reason: 'Invalid page number'
                });
                is400Sent = true;
            }
            else {
                this.query.setPage(page);
            }
        }
        if (is400Sent) {
            return;
        }
        tsDbHelper.Count(this.query).exec().subscribe(function (totalElementsCount) {
            var page = new Page();
            if (totalElementsCount === 0) {
                page.number = 0;
                page.size = 0;
                page.total = 0;
                page.totalElementsCount = 0;
                page.content = [];
                resp.status(200);
                resp.json(page);
                return;
            }
            var divResult = totalElementsCount / _this.query.getSize();
            var max = Math.floor(divResult);
            if (divResult === max) {
                max -= 1;
            }
            if (_this.query.getPage() <= max) {
                _this.query.exec().subscribe(function (qr) {
                    page.number = _this.query.getPage();
                    page.size = _this.query.getSize();
                    page.total = max + 1;
                    page.totalElementsCount = totalElementsCount;
                    if (qr.rows.length) {
                        var serializer = _this.getSerializerInstance(qr.rows.toArray(), true);
                        serializer.getData().subscribe(function (data) {
                            page.content = data;
                            resp.status(200);
                            resp.json(page);
                        }, function (err) {
                            resp.status(500);
                            resp.json({
                                reason: 'an error occurred while serializing retrieved datas',
                                details: err
                            });
                        });
                    }
                    else {
                        page.content = [];
                        resp.status(200);
                        resp.json(page);
                    }
                }, function (err) {
                    resp.status(500);
                    resp.json({
                        reason: 'Cannot fetch results',
                        details: err
                    });
                });
            }
            else {
                resp.status(400);
                resp.json({
                    reason: 'This page is out of range',
                    min: 0,
                    max: max
                });
            }
        }, function (err) {
            resp.status(500);
            resp.json({
                reason: 'Cannot count total "' + _this.model.name + '" count',
                details: err
            });
        });
    };
    ViewSet.prototype.retrieve = function (req, resp) {
        this.retrieveOrDelete(req, resp, true);
    };
    ViewSet.prototype.create = function (req, resp) {
        this.persist(req, resp, true);
    };
    ViewSet.prototype.update = function (req, resp) {
        this.persist(req, resp, false);
    };
    ViewSet.prototype.partialUpdate = function (req, resp) {
        this.persist(req, resp, false, true);
    };
    ViewSet.prototype.delete = function (req, resp) {
        this.retrieveOrDelete(req, resp, false);
    };
    ViewSet.prototype.retrieveOrDelete = function (req, resp, retreive) {
        var _this = this;
        var columns = tsDbHelper.ModelManager.getInstance().getModel(this.model).columnList;
        var clauses = {};
        var isInvalid = null;
        var missings = [];
        for (var _i = 0, columns_1 = columns; _i < columns_1.length; _i++) {
            var column = columns_1[_i];
            if (column.primaryKey) {
                if (req.params.hasOwnProperty(column.field)) {
                    if (column.autoIncrement && req.params[column.field] === null || req.params[column.field] === undefined) {
                        isInvalid = true;
                        missings.push(column.field);
                    }
                    else {
                        if (isInvalid === null) {
                            isInvalid = false;
                        }
                        clauses[column.name] = req.params[column.field];
                    }
                }
                else {
                    isInvalid = true;
                    missings.push(column.field);
                }
            }
        }
        if (isInvalid) {
            resp.status(400);
            resp.json({
                reason: 'A primary key is missing or invalid',
                invalidFields: missings
            });
        }
        else {
            if (retreive) {
                tsDbHelper.Select(this.model).where(clauses).exec().subscribe(function (qr) {
                    if (qr.rows.length === 0) {
                        resp.status(404);
                        resp.json({
                            reason: 'Ressource not found'
                        });
                    }
                    else {
                        var serializer = _this.getSerializerInstance(qr.rows.item(0), false);
                        serializer.getData().subscribe(function (data) {
                            resp.status(200);
                            resp.json(data);
                        }, function (err) {
                            resp.status(500);
                            resp.json({
                                reason: 'failed to serialize'
                            });
                        });
                    }
                }, function (err) {
                    resp.status(500);
                    resp.json({
                        reason: 'Retrieve element did fail',
                        details: err
                    });
                });
            }
            else {
                tsDbHelper.Delete(this.model).where(clauses).exec().subscribe(function () {
                    resp.status(204);
                    resp.send();
                }, function (err) {
                    resp.status(500);
                    resp.json({
                        reason: 'Delete query did fail',
                        details: err
                    });
                });
            }
        }
    };
    ViewSet.prototype.getSerializerInstance = function (target, many) {
        return this.serializer === tsDbHelper.AutoSerializer ?
            new this.serializer(this.model, target, many) :
            new this.serializer(target, many);
    };
    ViewSet.prototype.persist = function (req, resp, create, partial) {
        var _this = this;
        var data = req.body;
        if (data && !Array.isArray(data)) {
            var serializer = this.getSerializerInstance(data, false);
            if (partial) {
                serializer.setPartialParse(true);
            }
            serializer.getInstance().subscribe(function (instance) {
                return instance.checkIfShouldUpdateFromDatabase().subscribe(function (shouldUpdate) {
                    var persistObservable;
                    if (shouldUpdate && create) {
                        resp.status(400);
                        resp.json({
                            reason: 'Cannot create "' + _this.model.name + '", a model whith the same primary key already exists'
                        });
                    }
                    else if (!shouldUpdate && create) {
                        resp.status(400);
                        resp.json({
                            reason: 'Cannot update instance, primary keys are not valid or model should be created first'
                        });
                    }
                    else if (shouldUpdate) {
                        persistObservable = instance.update().map(function () {
                            return true;
                        });
                    }
                    else {
                        persistObservable = instance.insert().map(function () {
                            return true;
                        });
                    }
                    if (persistObservable) {
                        persistObservable.subscribe(function (didPersist) {
                            var savedSerializer = _this.getSerializerInstance(instance, false);
                            return savedSerializer.getData().subscribe(function (savedData) {
                                resp.status(201);
                                resp.json(savedData);
                            }, function (err) {
                                resp.status(500);
                                resp.json({
                                    reason: 'Deserialization of saved instance failed',
                                    details: err
                                });
                            });
                        }, function (err) {
                            resp.status(400);
                            resp.json({
                                reason: 'save instance did fail',
                                error: err
                            });
                        });
                    }
                }, function (err) {
                    resp.status(500);
                    resp.json({
                        reason: 'Fail to check if model already exists',
                        details: err
                    });
                });
            }, function (err) {
                if (err instanceof tsDbHelper.SerializerError) {
                    resp.status(400);
                    resp.json({
                        reason: 'one or many fields are not valid',
                        serializationMessage: err.message,
                        details: err.fieldErrors
                    });
                }
                else {
                    resp.status(500);
                    resp.json({
                        reason: 'Uncaught serialize error happened'
                    });
                }
            });
        }
        else {
            resp.status(400);
            resp.json({
                reason: 'Request body contains anything or an array. Only a json object is expected'
            });
        }
    };
    return ViewSet;
}());

var EndPointBuilder = /** @class */ (function () {
    function EndPointBuilder() {
    }
    EndPointBuilder.build = function (mViewset, key, routeConfig, endPointConfig, router) {
        var endpointDescription = endPointConfig.method.toUpperCase() + ' ' + routeConfig.path + endPointConfig.path
            + ' => ' + mViewset.name + '.' + key;
        Log.i(endpointDescription);
        return function (req, resp, next) {
            Log.d(endpointDescription);
            if (req.body) {
                Log.d('body: ' + JSON.stringify(req.body));
            }
            try {
                var viewset = new mViewset();
                EndPointBuilder.manageEndPointAuthFilter(req, resp, routeConfig, endPointConfig, viewset, key);
            }
            catch (e) {
                Log.e(e);
                resp.status(500);
                resp.json({
                    reason: 'Unhandled error',
                    details: e
                });
            }
        };
    };
    EndPointBuilder.manageEndPointError = function (req, resp, err) {
        if (err instanceof AuthFilterError) {
            resp.status(401);
            resp.json({
                reason: 'Invalid request authentication informations',
                details: err.message
            });
        }
        else {
            Log.e(err);
            resp.status(500);
            resp.json({
                reason: 'Unhandled error',
                details: err instanceof String ? err : err.toString()
            });
        }
    };
    EndPointBuilder.manageEndPointAuthFilter = function (req, resp, routeConfig, endPointConfig, viewset, key) {
        var user;
        var err;
        EndPointBuilder.callEndPointAuthFilter(req, routeConfig, endPointConfig)
            .finally(function () {
            if (err) {
                EndPointBuilder.manageEndPointError(req, resp, err);
            }
            else {
                if (user) {
                    req.user = user;
                }
                EndPointBuilder.manageEndPointAuthPermission(req, resp, routeConfig, endPointConfig, viewset, key);
            }
        })
            .subscribe(function (iUser) {
            user = iUser;
        }, function (authFilterErr) {
            Log.e(err);
            err = authFilterErr;
        });
    };
    EndPointBuilder.callEndPointAuthFilter = function (req, routeConfig, endPointConfig) {
        var edpFilters = [];
        var authFilter = endPointConfig.authFilter || routeConfig.authFilter;
        if (authFilter) {
            Log.d(endPointConfig.authFilter ? endPointConfig.authFilter.constructor.name : 'no enpoint filter');
            Log.d(routeConfig.authFilter ? routeConfig.authFilter.constructor.name : 'no route filter');
            edpFilters.push(authFilter);
        }
        if (!edpFilters.length) {
            edpFilters = edpFilters.concat(EndPointBuilder.authFilters);
        }
        if (routeConfig.addAuthFilters) {
            edpFilters = edpFilters.concat(routeConfig.addAuthFilters);
        }
        if (endPointConfig.addAuthFilters) {
            edpFilters = edpFilters.concat(endPointConfig.addAuthFilters);
        }
        if (!edpFilters.length) {
            var u = new EndPointBuilder.userModel();
            u.isAnonymous = true;
            return rxjs_Observable.Observable.from([u]);
        }
        else {
            return EndPointBuilder.scanAuthFilters(req, edpFilters);
        }
    };
    EndPointBuilder.scanAuthFilters = function (req, edpFilters, index) {
        if (index === void 0) { index = 0; }
        return edpFilters[index].filter(req, EndPointBuilder.userModel).switchMap(function (user) {
            if (!user && index < edpFilters.length - 1) {
                return EndPointBuilder.scanAuthFilters(req, edpFilters, index + 1);
            }
            else {
                return rxjs_Observable.Observable.create(function (observer) {
                    observer.next(user);
                    observer.complete();
                });
            }
        });
    };
    EndPointBuilder.manageEndPointAuthPermission = function (req, resp, routeConfig, endPointConfig, viewset, key) {
        var result;
        var err;
        EndPointBuilder.callAuthPermissions(req, routeConfig, endPointConfig)
            .finally(function () {
            if (err) {
                EndPointBuilder.manageEndPointError(req, resp, err);
            }
            else if (!result || result.isOk) {
                EndPointBuilder.manageEndPointPermissions(req, resp, routeConfig, endPointConfig, viewset, key);
            }
            else {
                resp.status(401);
                resp.json({
                    reason: 'Request not authenticated',
                    details: result ? result.rejectionReason : ''
                });
            }
        }).subscribe(function (permissionResult) {
            result = permissionResult;
        }, function (authPermissionErr) {
            err = authPermissionErr;
        });
    };
    EndPointBuilder.callAuthPermissions = function (req, routeConfig, endPointConfig) {
        var authPermission = endPointConfig.authPermission || routeConfig.authPermission || EndPointBuilder.defaultAuthPermission;
        if (authPermission) {
            return authPermission.check(req, EndPointBuilder.userModel);
        }
        else {
            return rxjs_Observable.Observable.create(function (observer) {
                observer.next(new PermissionResult(true));
                observer.complete();
            });
        }
    };
    EndPointBuilder.manageEndPointPermissions = function (req, resp, routeConfig, endPointConfig, viewset, key) {
        var result;
        var err;
        EndPointBuilder.callPermissions(req, routeConfig, endPointConfig)
            .finally(function () {
            if (err) {
                EndPointBuilder.manageEndPointError(req, resp, err);
            }
            else if (!result || result.isOk) {
                try {
                    viewset[key](req, resp);
                }
                catch (e) {
                    EndPointBuilder.manageEndPointError(req, resp, e);
                }
            }
            else {
                resp.status(403);
                resp.json({
                    reason: 'Request not authorized',
                    details: result ? result.rejectionReason : ''
                });
            }
        }).subscribe(function (permissionResult) {
            result = permissionResult;
        }, function (error) {
            err = error;
        });
    };
    EndPointBuilder.callPermissions = function (req, routeConfig, endPointConfig) {
        var permissionFilters = [];
        var permissionFilter = endPointConfig.permissionFilter || routeConfig.permissionFilter || EndPointBuilder.defaultPermissionFilter;
        if (permissionFilter) {
            permissionFilters.push(permissionFilter);
        }
        if (routeConfig.addPermissionFilters) {
            permissionFilters = permissionFilters.concat(routeConfig.addPermissionFilters);
        }
        if (endPointConfig.addPermissionFilters) {
            permissionFilters = permissionFilters.concat(endPointConfig.addPermissionFilters);
        }
        if (permissionFilters.length) {
            return EndPointBuilder.scanPermissionFilters(req, permissionFilters, 0);
        }
        else {
            return rxjs_Observable.Observable.empty();
        }
    };
    EndPointBuilder.scanPermissionFilters = function (req, permissionFilters, index) {
        return permissionFilters[index].filter(req, EndPointBuilder.userModel).switchMap(function (result) {
            if (result.isOk && index < permissionFilters.length - 1) {
                return EndPointBuilder.scanPermissionFilters(req, permissionFilters, index + 1);
            }
            else {
                return rxjs_Observable.Observable.create(function (observer) {
                    observer.next(result);
                    observer.complete();
                });
            }
        });
    };
    EndPointBuilder.authFilters = [];
    return EndPointBuilder;
}());

var Express = express;
// Creates and configures an ExpressJS web server.
var TsRestFrameworkApp = /** @class */ (function () {
    // Run configuration methods on the Express instance.
    function TsRestFrameworkApp() {
        this.pathBase = '/';
        this.express = Express();
    }
    // Configure Express middleware.
    TsRestFrameworkApp.prototype.middleware = function () {
        this.express.use(function (req, resp, next) {
            // log things
            next();
        });
        this.express.use(bodyParser.json());
        this.express.use(bodyParser.urlencoded({ extended: false }));
    };
    TsRestFrameworkApp.prototype.setAuthenticator = function (method, authPath, authenticator) {
        if (authenticator && authPath) {
            var authRouter = express.Router();
            authRouter.use(bodyParser.json());
            authRouter.use(bodyParser.urlencoded({ extended: false }));
            if (TsRestFrameworkApp.methods.indexOf(method.toLowerCase()) !== -1) {
                authRouter[method]('/', function (req, resp, next) {
                    authenticator.authenticate(req, resp, EndPointBuilder.userModel);
                });
            }
            else {
                throw new Error('Try to add auth point with unknown method value: ' + method);
            }
            this.express.use(authPath, authRouter);
        }
        if (authenticator && authenticator.authFilter) {
            EndPointBuilder.authFilters.push(authenticator.authFilter);
        }
        EndPointBuilder.defaultAuthPermission = new IsAutenticatedPermission();
    };
    TsRestFrameworkApp.prototype.setDefaultAuthPermission = function (authPermission) {
        EndPointBuilder.defaultAuthPermission = authPermission;
    };
    TsRestFrameworkApp.prototype.setDefaultPermissionFilter = function (permissionFilter) {
        EndPointBuilder.defaultPermissionFilter = permissionFilter;
    };
    TsRestFrameworkApp.prototype.setUserModel = function (userModel) {
        EndPointBuilder.userModel = userModel;
    };
    TsRestFrameworkApp.prototype.setPathBase = function (base) {
        this.pathBase = base;
        if (this.pathBase[0] !== '/') {
            this.pathBase = '/' + this.pathBase;
        }
    };
    TsRestFrameworkApp.prototype.setDbConfig = function (config) {
        this.dbConfig = config;
    };
    // Configure API endpoints.
    TsRestFrameworkApp.prototype.import = function (view, parentPath) {
        if (parentPath === void 0) { parentPath = this.pathBase; }
        if (RouteManager.getInstance().isRegisteredNamespace(view)) {
            this.importNamespace(view, this.pathBase);
        }
        else if (RouteManager.getInstance().isRegisteredViewset(view)) {
            this.importViewset(view, this.pathBase);
        }
        else {
            throw new Error('"' + view.name + '" is not registered did you miss "@Namespace" or "@Route" annotation ?');
        }
        return this;
    };
    TsRestFrameworkApp.prototype.importViewset = function (mViewset, parentPath) {
        var bundle = RouteManager.getInstance().getRouteBundle(mViewset);
        if (!bundle) {
            throw new Error('ViewSet "' + mViewset.name + '" has no configuration, did you miss "@Route" decorator on this ViewSet ?');
        }
        var router = express.Router();
        var routePath = this.appendPath(parentPath, bundle.config.path);
        if (bundle.config.middlewares && bundle.config.middlewares.length) {
            for (var _i = 0, _a = bundle.config.middlewares; _i < _a.length; _i++) {
                var requestHandler = _a[_i];
                router.use(requestHandler);
            }
        }
        else {
            router.use(bodyParser.json());
            router.use(bodyParser.urlencoded({ extended: false }));
        }
        for (var key in bundle.routes) {
            if (bundle.routes.hasOwnProperty(key)) {
                var endPointConfig = bundle.routes[key];
                this.addRoute(mViewset, key, bundle.config, endPointConfig, router);
            }
        }
        this.express.use(routePath, router);
    };
    TsRestFrameworkApp.prototype.addRoute = function (mViewset, key, routeConfig, endPointConfig, router) {
        if (!endPointConfig.path || endPointConfig.path[0] !== '/') {
            endPointConfig.path = '/' + endPointConfig.path;
        }
        endPointConfig.method = endPointConfig.method || 'get';
        var endPoint = EndPointBuilder.build(mViewset, key, routeConfig, endPointConfig, router);
        if (TsRestFrameworkApp.methods.indexOf(endPointConfig.method.toLowerCase()) !== -1) {
            router[endPointConfig.method](endPointConfig.path, endPoint);
        }
        else {
            throw new Error('Try to add end point with unknown method');
        }
    };
    TsRestFrameworkApp.prototype.importNamespace = function (mNamespace, parentPath) {
        var namespaceConfig = RouteManager.getInstance().getNamespaceConfiguration(mNamespace);
        namespaceConfig.path = namespaceConfig.path || '';
        var childPath = this.appendPath(parentPath, namespaceConfig.path);
        for (var _i = 0, _a = namespaceConfig.views; _i < _a.length; _i++) {
            var view = _a[_i];
            this.import(view, childPath);
        }
    };
    TsRestFrameworkApp.prototype.appendPath = function (parentPath, childPath) {
        var completePath;
        if (!childPath) {
            completePath = parentPath;
        }
        else if (parentPath[parentPath.length - 1] === '/' && childPath[0] === '/') {
            completePath = parentPath + childPath.substr(1, childPath.length);
        }
        else if (parentPath[parentPath.length - 1] === '/' || childPath[0] === '/') {
            completePath = parentPath + childPath;
        }
        else {
            completePath = parentPath + '/' + childPath;
        }
        return completePath;
    };
    TsRestFrameworkApp.prototype.start = function () {
        tsDbHelper.QueryManager.init(this.dbConfig);
        var port = normalizePort(process.env.PORT || 3000);
        this.express.set('port', port);
        var server = http.createServer(this.express);
        server.listen(port);
        server.on('error', onError);
        server.on('listening', onListening);
        function normalizePort(val) {
            var nPort = (typeof val === 'string') ? parseInt(val, 10) : val;
            if (isNaN(nPort)) {
                return val;
            }
            else if (nPort >= 0) {
                return nPort;
            }
            else {
                return false;
            }
        }
        function onError(error) {
            if (error.syscall !== 'listen') {
                throw error;
            }
            var bind = (typeof port === 'string') ? 'Pipe ' + port : 'Port ' + port;
            switch (error.code) {
                case 'EACCES':
                    Log.e(bind + " requires elevated privileges");
                    process.exit(1);
                    break;
                case 'EADDRINUSE':
                    Log.e(bind + " is already in use");
                    process.exit(1);
                    break;
                default:
                    throw error;
            }
        }
        function onListening() {
            var addr = server.address();
            var bind = (typeof addr === 'string') ? "pipe " + addr : "port " + addr.port;
            Log.i("Listening on " + bind);
        }
    };
    TsRestFrameworkApp.methods = ['get', 'post', 'put', 'patch', 'delete', 'options', 'head', 'connect', 'trace', 'all'];
    return TsRestFrameworkApp;
}());

/**
 * @module
 * @description
 * Entry point for all public APIs of the async local storage package.
 */

exports.IsAutenticatedPermission = IsAutenticatedPermission;
exports.AnonymousPermission = AnonymousPermission;
exports.NoAuthenticator = NoAuthenticator;
exports.NoAuthFilter = NoAuthFilter;
exports.JWTFilter = JWTFilter;
exports.JWTConfiguration = JWTConfiguration;
exports.JWTAuthenticator = JWTAuthenticator;
exports.BasicAuthFilter = BasicAuthFilter;
exports.BasicAuthAuthenticator = BasicAuthAuthenticator;
exports.PermissionResult = PermissionResult;
exports.IUser = IUser;
exports.AuthFilterError = AuthFilterError;
exports.Route = Route;
exports.Namespace = Namespace;
exports.All = All;
exports.Connect = Connect;
exports.Delete = Delete$1;
exports.Get = Get;
exports.Head = Head;
exports.Options = Options;
exports.Patch = Patch;
exports.Post = Post;
exports.Put = Put;
exports.RouteConfigurator = RouteConfigurator;
exports.NamespaceConfigurator = NamespaceConfigurator;
exports.EndPointConfigurator = EndPointConfigurator;
exports.ViewSet = ViewSet;
exports.TsRestFrameworkApp = TsRestFrameworkApp;
