import { Delete, ModelManager, SerializerError, Count, Select, AutoSerializer } from 'ts-db-helper';
import { Page } from '../public-api/page';
var ViewSet = /** @class */ (function () {
    function ViewSet() {
        if (!this.query) {
            this.query = Select(this.model).setSize(50);
        }
        if (!this.serializer) {
            this.serializer = AutoSerializer;
        }
    }
    ViewSet.prototype.search = function (req, resp) {
    };
    ViewSet.prototype.list = function (req, resp) {
        var _this = this;
        var is400Sent = false;
        if (req.query.size) {
            var size = parseInt(req.query.pageSize, 10);
            if (isNaN(size)) {
                resp.status(400);
                resp.json({
                    reason: 'Invalid page size'
                });
                is400Sent = true;
            }
            else {
                this.query.setSize(size);
            }
        }
        if (req.query.page) {
            var page = parseInt(req.query.pageNumber, 10);
            if (isNaN(page)) {
                resp.status(400);
                resp.json({
                    reason: 'Invalid page number'
                });
                is400Sent = true;
            }
            else {
                this.query.setPage(page);
            }
        }
        if (is400Sent) {
            return;
        }
        Count(this.query).exec().subscribe(function (totalElementsCount) {
            var page = new Page();
            if (totalElementsCount === 0) {
                page.number = 0;
                page.size = 0;
                page.total = 0;
                page.totalElementsCount = 0;
                page.content = [];
                resp.status(200);
                resp.json(page);
                return;
            }
            var divResult = totalElementsCount / _this.query.getSize();
            var max = Math.floor(divResult);
            if (divResult === max) {
                max -= 1;
            }
            if (_this.query.getPage() <= max) {
                _this.query.exec().subscribe(function (qr) {
                    page.number = _this.query.getPage();
                    page.size = _this.query.getSize();
                    page.total = max + 1;
                    page.totalElementsCount = totalElementsCount;
                    if (qr.rows.length) {
                        var serializer = _this.getSerializerInstance(qr.rows.toArray(), true);
                        serializer.getData().subscribe(function (data) {
                            page.content = data;
                            resp.status(200);
                            resp.json(page);
                        }, function (err) {
                            resp.status(500);
                            resp.json({
                                reason: 'an error occurred while serializing retrieved datas',
                                details: err
                            });
                        });
                    }
                    else {
                        page.content = [];
                        resp.status(200);
                        resp.json(page);
                    }
                }, function (err) {
                    resp.status(500);
                    resp.json({
                        reason: 'Cannot fetch results',
                        details: err
                    });
                });
            }
            else {
                resp.status(400);
                resp.json({
                    reason: 'This page is out of range',
                    min: 0,
                    max: max
                });
            }
        }, function (err) {
            resp.status(500);
            resp.json({
                reason: 'Cannot count total "' + _this.model.name + '" count',
                details: err
            });
        });
    };
    ViewSet.prototype.retrieve = function (req, resp) {
        this.retrieveOrDelete(req, resp, true);
    };
    ViewSet.prototype.create = function (req, resp) {
        this.persist(req, resp, true);
    };
    ViewSet.prototype.update = function (req, resp) {
        this.persist(req, resp, false);
    };
    ViewSet.prototype.partialUpdate = function (req, resp) {
        this.persist(req, resp, false, true);
    };
    ViewSet.prototype.delete = function (req, resp) {
        this.retrieveOrDelete(req, resp, false);
    };
    ViewSet.prototype.retrieveOrDelete = function (req, resp, retreive) {
        var _this = this;
        var columns = ModelManager.getInstance().getModel(this.model).columnList;
        var clauses = {};
        var isInvalid = null;
        var missings = [];
        for (var _i = 0, columns_1 = columns; _i < columns_1.length; _i++) {
            var column = columns_1[_i];
            if (column.primaryKey) {
                if (req.params.hasOwnProperty(column.field)) {
                    if (column.autoIncrement && req.params[column.field] === null || req.params[column.field] === undefined) {
                        isInvalid = true;
                        missings.push(column.field);
                    }
                    else {
                        if (isInvalid === null) {
                            isInvalid = false;
                        }
                        clauses[column.name] = req.params[column.field];
                    }
                }
                else {
                    isInvalid = true;
                    missings.push(column.field);
                }
            }
        }
        if (isInvalid) {
            resp.status(400);
            resp.json({
                reason: 'A primary key is missing or invalid',
                invalidFields: missings
            });
        }
        else {
            if (retreive) {
                Select(this.model).where(clauses).exec().subscribe(function (qr) {
                    if (qr.rows.length === 0) {
                        resp.status(404);
                        resp.json({
                            reason: 'Ressource not found'
                        });
                    }
                    else {
                        var serializer = _this.getSerializerInstance(qr.rows.item(0), false);
                        serializer.getData().subscribe(function (data) {
                            resp.status(200);
                            resp.json(data);
                        }, function (err) {
                            resp.status(500);
                            resp.json({
                                reason: 'failed to serialize'
                            });
                        });
                    }
                }, function (err) {
                    resp.status(500);
                    resp.json({
                        reason: 'Retrieve element did fail',
                        details: err
                    });
                });
            }
            else {
                Delete(this.model).where(clauses).exec().subscribe(function () {
                    resp.status(204);
                    resp.send();
                }, function (err) {
                    resp.status(500);
                    resp.json({
                        reason: 'Delete query did fail',
                        details: err
                    });
                });
            }
        }
    };
    ViewSet.prototype.getSerializerInstance = function (target, many) {
        return this.serializer === AutoSerializer ?
            new this.serializer(this.model, target, many) :
            new this.serializer(target, many);
    };
    ViewSet.prototype.persist = function (req, resp, create, partial) {
        var _this = this;
        var data = req.body;
        if (data && !Array.isArray(data)) {
            var serializer = this.getSerializerInstance(data, false);
            if (partial) {
                serializer.setPartialParse(true);
            }
            serializer.getInstance().subscribe(function (instance) {
                return instance.checkIfShouldUpdateFromDatabase().subscribe(function (shouldUpdate) {
                    var persistObservable;
                    if (shouldUpdate && create) {
                        resp.status(400);
                        resp.json({
                            reason: 'Cannot create "' + _this.model.name + '", a model whith the same primary key already exists'
                        });
                    }
                    else if (!shouldUpdate && create) {
                        resp.status(400);
                        resp.json({
                            reason: 'Cannot update instance, primary keys are not valid or model should be created first'
                        });
                    }
                    else if (shouldUpdate) {
                        persistObservable = instance.update().map(function () {
                            return true;
                        });
                    }
                    else {
                        persistObservable = instance.insert().map(function () {
                            return true;
                        });
                    }
                    if (persistObservable) {
                        persistObservable.subscribe(function (didPersist) {
                            var savedSerializer = _this.getSerializerInstance(instance, false);
                            return savedSerializer.getData().subscribe(function (savedData) {
                                resp.status(201);
                                resp.json(savedData);
                            }, function (err) {
                                resp.status(500);
                                resp.json({
                                    reason: 'Deserialization of saved instance failed',
                                    details: err
                                });
                            });
                        }, function (err) {
                            resp.status(400);
                            resp.json({
                                reason: 'save instance did fail',
                                error: err
                            });
                        });
                    }
                }, function (err) {
                    resp.status(500);
                    resp.json({
                        reason: 'Fail to check if model already exists',
                        details: err
                    });
                });
            }, function (err) {
                if (err instanceof SerializerError) {
                    resp.status(400);
                    resp.json({
                        reason: 'one or many fields are not valid',
                        serializationMessage: err.message,
                        details: err.fieldErrors
                    });
                }
                else {
                    resp.status(500);
                    resp.json({
                        reason: 'Uncaught serialize error happened'
                    });
                }
            });
        }
        else {
            resp.status(400);
            resp.json({
                reason: 'Request body contains anything or an array. Only a json object is expected'
            });
        }
    };
    return ViewSet;
}());
export { ViewSet };
//# sourceMappingURL=viewset.js.map