import { JWTFilter } from './jwt.filter';
import * as jwt from 'jsonwebtoken';
var JWTAuthenticator = /** @class */ (function () {
    function JWTAuthenticator(config) {
        this.config = config;
        this.authFilter = new JWTFilter(this.config);
    }
    JWTAuthenticator.prototype.authenticate = function (req, resp, userModel) {
        var _this = this;
        var body = req.body;
        if (body && body.username && body.password) {
            userModel.authenticate(body.username, body.password).subscribe(function (user) {
                if (!user || user.isAnonymous) {
                    resp.status(400);
                    resp.json({
                        reason: 'Invalid credentials'
                    });
                }
                else {
                    var userData = _this.config.serializeUser(user);
                    var config = {};
                    if (_this.config.algorithm) {
                        config.algorithm = _this.config.algorithm;
                    }
                    if (_this.config.audience) {
                        config.audience = _this.config.audience;
                    }
                    if (_this.config.issuer) {
                        config.issuer = _this.config.issuer;
                    }
                    try {
                        var token = jwt.sign({ user: userData }, _this.config.secret, config);
                        resp.status(200);
                        resp.json({
                            token: token
                        });
                    }
                    catch (err) {
                        resp.status(500);
                        resp.json({
                            reason: 'Token creation did fail',
                            details: err.message || JSON.stringify(err)
                        });
                    }
                }
            }, function (err) {
                var content = { reason: '' };
                if (err instanceof String) {
                    content.reason = err;
                }
                else if (err && err.message) {
                    content.reason = err.message;
                }
                else {
                    content.reason = JSON.stringify(err);
                }
                resp.status(500);
                resp.json(content);
            });
        }
        else {
            resp.status(400);
            resp.json({
                reason: 'Username, password or both are missing in message body'
            });
        }
    };
    return JWTAuthenticator;
}());
export { JWTAuthenticator };
//# sourceMappingURL=jwt.authenticator.js.map