import { Observable } from 'rxjs/Observable';
import { AuthFilterError } from '../../errors/auth-filter.error';
import { StringUtils } from '../../utils/string.utils';
import * as jwt from 'jsonwebtoken';
import { Log } from '../../internal/log';
var JWTFilter = /** @class */ (function () {
    function JWTFilter(config) {
        this.config = config;
    }
    JWTFilter.prototype.filter = function (req, userModel) {
        var _this = this;
        return Observable.create(function (observer) {
            var header = req.headers[_this.config.httpHeader.toLowerCase()];
            var token = null;
            var err = null;
            if (header && StringUtils.isString(header)) {
                if (_this.config.httpHeaderAuthType) {
                    var headerParts = header.split(' ');
                    if (headerParts.length === 2 && headerParts[0].toUpperCase() === _this.config.httpHeaderAuthType.toUpperCase()) {
                        token = headerParts[1];
                    }
                    else {
                        err = 'Token header do not conform to "' + _this.config.httpHeaderAuthType + ' <token>"';
                    }
                }
                else {
                    token = header;
                }
            }
            if (token) {
                var decodeConfig = {};
                if (_this.config.audience) {
                    decodeConfig.audience = _this.config.audience;
                }
                if (_this.config.algorithm) {
                    decodeConfig.algorithm = _this.config.algorithm;
                }
                if (_this.config.issuer) {
                    decodeConfig.algorithm = _this.config.algorithm;
                }
                jwt.verify(token, _this.config.secret, decodeConfig, function (error, decoded) {
                    if (error) {
                        Log.e(error.message || JSON.stringify(error));
                        observer.error(new AuthFilterError(error.name + ': ' + error.message));
                    }
                    else if (decoded instanceof String) {
                        observer.error(new AuthFilterError('Token has an invalid payload'));
                    }
                    else if (decoded.user) {
                        try {
                            var user = _this.config.deserializeUser(decoded.user);
                            observer.next(user);
                        }
                        catch (e) {
                            Log.e(e.message || JSON.stringify(e));
                            observer.error(new AuthFilterError('Token contains invalid user'));
                        }
                    }
                    observer.complete();
                });
            }
            else {
                if (err) {
                    observer.error(new AuthFilterError(err));
                }
                else {
                    var user = new userModel();
                    user.isAnonymous = true;
                    observer.next(user);
                }
                observer.complete();
            }
        });
    };
    return JWTFilter;
}());
export { JWTFilter };
//# sourceMappingURL=jwt.filter.js.map